<?php

namespace FiloBlu\Job\Controller;

use FiloBlu\Storelocator\Model\StoresFactory;
use Magento\Framework\App\Action\Forward;
use Magento\Framework\App\ActionFactory;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\RouterInterface;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\ScopeInterface;

class Router implements RouterInterface
{
    /**
     * Enabled config path
     */
    const XML_PATH_ENABLED = 'filoblu_job_section/general/enabled';
    /**
     * Custom URL
     */
    const XML_PATH_CUSTOM_URL = 'filoblu_job_section/general/custom_url';
    /**
     * @var ActionFactory
     */
    protected $actionFactory;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var StoresFactory
     */
    protected $storesFactory;

    /**
     * Router constructor.
     *
     * @param ActionFactory $actionFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param StoresFactory $storesFactory
     */
    public function __construct(
        ActionFactory $actionFactory,
        ScopeConfigInterface $scopeConfig,
        StoresFactory $storesFactory
    ) {
        $this->actionFactory = $actionFactory;
        $this->scopeConfig = $scopeConfig;
        $this->storesFactory = $storesFactory;
    }

    /**
     * @param RequestInterface $request
     * @return ActionInterface
     */
    public function match(RequestInterface $request)
    {
        if (!$this->scopeConfig->isSetFlag(self::XML_PATH_ENABLED, ScopeInterface::SCOPE_STORE)) {
            return null;
        }

        $customUrl = trim($this->scopeConfig->getValue(self::XML_PATH_CUSTOM_URL, ScopeInterface::SCOPE_STORE), '/');

        if ($customUrl === null || empty($customUrl)) {
            return null;
        }

        $path = trim($request->getPathInfo(), '/');

        if ($path !== $customUrl) {
            return null;
        }

        $request->setAlias(UrlInterface::REWRITE_REQUEST_PATH_ALIAS, $path);
        $request->setPathInfo('/job/index/index');

        return $this->actionFactory->create(Forward::class);
    }
}
