<?php

namespace FiloBlu\Job\Model\Mail;

use Magento\Framework\App\TemplateTypesInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Mail\MailMessageInterface;
use Magento\Framework\Mail\MessageInterfaceFactory;
use Magento\Framework\Mail\Template\FactoryInterface;
use Magento\Framework\Mail\Template\SenderResolverInterface;
use Magento\Framework\Mail\TransportInterfaceFactory;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Phrase;
use Zend\Mime\MessageFactory as MimeMessageFactory;
use Zend\Mime\PartFactory;

/**
 * Class TransportBuilder
 * @package FiloBlu\Job\Model\Mail
 */
class TransportBuilder extends \Magento\Framework\Mail\Template\TransportBuilder
{
    /**
     * @var Message
     */
    protected $message;
    /**
     * @var PartFactory
     */
    protected $partFactory;
    /**
     * @var MimeMessageFactory
     */
    protected $mimeMessageFactory;

    /**
     * TransportBuilder constructor.
     * @param FactoryInterface $templateFactory
     * @param MailMessageInterface $message
     * @param SenderResolverInterface $senderResolver
     * @param ObjectManagerInterface $objectManager
     * @param TransportInterfaceFactory $mailTransportFactory
     * @param PartFactory $partFactory
     * @param MimeMessageFactory $mimeMessageFactory
     * @param MessageInterfaceFactory $messageFactory
     */
    public function __construct(
        FactoryInterface $templateFactory,
        MailMessageInterface $message,
        SenderResolverInterface $senderResolver,
        ObjectManagerInterface $objectManager,
        TransportInterfaceFactory $mailTransportFactory,
        PartFactory $partFactory,
        MimeMessageFactory $mimeMessageFactory,
        MessageInterfaceFactory $messageFactory = null
    ) {
        parent::__construct($templateFactory, $message, $senderResolver, $objectManager, $mailTransportFactory, $messageFactory);
        $this->partFactory = $partFactory;
        $this->mimeMessageFactory = $mimeMessageFactory;
    }

    /**
     * Add an attachment to the message.
     *
     * @param string $file
     * @param string $fileName
     * @return $this
     */
    public function addAttachment($file, $fileName)
    {
        $data = file_get_contents($file);
        $mime = 'application/octet-stream';

        if (function_exists('finfo_open')) {
            $finfo = finfo_open(FILEINFO_MIME);
            $mime = finfo_buffer($finfo, $data, FILEINFO_MIME);
            finfo_close($finfo);
        } elseif (function_exists('mime_content_type')) {
            $mime = mime_content_type($file);
        }

        $this->message->setBodyAttachment($data, $fileName, $mime);

        return $this;
    }

    /**
     * After all parts are set, add them to message body.
     *
     * @return $this
     * @throws LocalizedException
     */
    protected function prepareMessage()
    {
        $template = $this->getTemplate();
        $body = $template->processTemplate();

        switch ($template->getType()) {
            case TemplateTypesInterface::TYPE_TEXT:
                $this->message->setBodyText($body);

                break;

            case TemplateTypesInterface::TYPE_HTML:
                $this->message->setBodyHtml($body);
                break;

            default:
                throw new LocalizedException(
                    new Phrase('Unknown template type')
                );
        }
        $this->message->setPartsToBody();
        $this->message->setSubject(html_entity_decode($template->getSubject(), ENT_QUOTES));
        return $this;
    }
}